%
%This code does generation of ASIFT descriptors and also obtains the ASIFT 
%sparse codes. The codes are used to train the model using AdaBoost and
%then test it using the same. A variety of libraries are used. Please
%refer the "Instructions.txt" for further information.

%Inputs: None
%Outputs: Training and Testing hypothesis and Percentage Accuracy(Results)

%Author: Naveen Kulkarni
%Group: Visual Processing and Representation Group, ASU
%October 11 2010
%

function [train_hypothesis test_hypothesis Final_hypothesis Results] = Adaboost_M1();

clear all; close all; clc;
addpath('');
% -------------------------------------------------------------------------


train_samples_len  = 30;           % training sample size for each class

Caltech101 = 1;
addpath('Extract_ASIFT');
addpath('Generate_AfSpCode');
addpath('Dictionary_Learning/SPAMS/release_win');
addpath('Liblinear');
addpath('Liblinear/matlab');        % Liblinear package is used.

% IMAGE CLASSIFICATION USING AFFINE SPARSE CODES.
% -------------------------------------------------------------------------

if Caltech101
% directory for the image database  
caltech101 = 'Caltech101';     
% Folder path for ASIFT descriptors
asift_descr = 'ASIFT_DESC/Caltech101';     
class_size = 102;
Dataset_size = 9144;
train_size = class_size*train_samples_len;
test_size = Dataset_size - train_size;
ASIFT_descriptors(caltech101, asift_descr);
else
% directory for the image database  
caltech256 = 'Caltech256';    
% Folder path for ASIFT descriptors
asift_descr = 'ASIFT_DESC/Caltech256';       
class_size = 257;
Dataset_size = 30607;
train_size = class_size*train_samples_len;
test_size = Dataset_size - train_size; 
ASIFT_descriptors(caltech256, asift_descr);    
end

%Get the ASIFT descriptors path to generate Asift sparse codes.
directory_struct = get_namespath(asift_descr); 
if isempty(directory_struct),
    error('Error, Get Back!');
end

% Get the Asift sparse codes to be subjected to AdaBoost
[directory_struct,feat_len] = ASIFT_max_pooling(directory_struct,class_size);


% load('fdatabase_LASSO.mat');

% feat_len = 21504;
% directory_struct = fdatabase;
% directory_struct.folder_path = fdatabase.path;

fn = @svm_train_test;
T = 3;
Repeat_For = 10;
  
train_hypothesis = zeros(train_size,T,Repeat_For);
test_hypothesis  = zeros(test_size,T,Repeat_For);
alpha_multi = zeros(T,Repeat_For);
Results = zeros(Repeat_For,1);
Final_hypothesis = zeros(test_size,T,Repeat_For);

fprintf('\n Training and Testing Using AdaBoost...\n');
class_index = unique(directory_struct.label);
nclass = length(class_index);

for rounds = 1:Repeat_For
    fprintf('Iteration : %d...\n', rounds);
    training_indices = [];
    test_indices = [];
    
    for jind = 1:nclass,
        index_val = find(directory_struct.label == class_index(jind));
        num = length(index_val);        
        idx_rand = randperm(num);        
        training_indices = [training_indices; index_val(idx_rand(1:train_samples_len))];
        test_indices = [test_indices; index_val(idx_rand(train_samples_len+1:end))];
    end
    
    fprintf('Training number: %d\n', length(training_indices));
    fprintf('Testing number:%d\n', length(test_indices));
    
    % All the training features 
    train_features = zeros(length(training_indices), feat_len);
    training_label = zeros(length(training_indices), 1);
    
    for jind = 1:length(training_indices),
        folder_p = directory_struct.folder_path{training_indices(jind)};
        load(folder_p, 'label','Asift_desc');
        train_features(jind, :) = Asift_desc';
        training_label(jind) = label;
    end  
    
     % All the testing features
        
    test_features = zeros(test_size, feat_len);
    test_label = zeros(test_size, 1);
    
    block_index = test_indices(1:test_size); 
            
            
    for kind = 1:test_size,
        folder_p = directory_struct.folder_path{block_index(kind)};
        load(folder_p, 'label','Asift_desc');
        test_features(kind, :) = Asift_desc';
        test_label(kind) = label;
    end 

    D_multi_n = ones(train_size,1);
    D_multi_n = D_multi_n/train_size;
    Pr = D_multi_n;
    Hf = zeros(test_size,1);
    
    for t=1:T

        [h_t_n werr_multi_n1  mod_n] = fn([],train_features,training_label,D_multi_n);
        train_hypothesis(:,t,rounds) = h_t_n;

        [hlatest] = predict(test_label, sparse(test_features), mod_n);
        test_hypothesis(:,t,rounds) = hlatest;

        h_multi_n = h_t_n;
        for ll = 1:class_size
            for k = 1:train_samples_len
                if(h_multi_n(train_samples_len*ll - train_samples_len + k) ~=ll); 
                   h_multi_n(train_samples_len*ll -train_samples_len + k) = 1;
                else
                    h_multi_n(train_samples_len*ll -train_samples_len + k) = 0;
                end
            end
        end

        werr_multi = sum(Pr(h_t_n ~= training_label));

        alpha_multi(t,rounds) = werr_multi/(1-werr_multi);

        for i=1:train_size
            D_multi_n(i)= D_multi_n(i)*(alpha_multi(t,rounds)^(1 - (h_multi_n(i) )));
        end

        Pr = D_multi_n;%/sum(D_multi_n);
        
        for i=1:test_size
            if(test_hypothesis(i,t,rounds) == test_label(i)); 
               Final_hypothesis(i,t,rounds)= 1;
            else
                Final_hypothesis(i,t,rounds) = 0;
            end
        end
        
        Hf = Hf + log(1/alpha_multi(t,rounds))*Final_hypothesis(:,t,rounds);

    end
        
        Results(rounds) = size(find(Hf > 0),1);
    
end
fprintf('Percentage Classification Accuracy = %d...\n',(100*mean(Results))/test_size);
DONE = 1;